import java.awt.BorderLayout;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;

import java.awt.image.BufferedImage;

import javax.swing.ButtonGroup;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/*
 * Created on 02.07.2004 by Andreas Pleu
 * Rewritten 14.06.2005 by Heiko Drewes
 */

/**
 * 
 * @author Andreas Pleu / Heiko Drewes
 */

public class ImageFrame extends JFrame implements MouseListener {

	private int OrgSizeX = 2000;
	private int OrgSizeY = 600;
	private int DestSizeX = 1000;
	private int DestSizeY = 300;
	private int lenssize = 100;
	private boolean showLens = false;
	private int focusX = 500;
	private int focusY = 150;
	private boolean fisheye = false;
	
	private float factor = DestSizeX / OrgSizeX;

	private static String imgPath = "long-pic3.jpg";
	private BufferedImage originalImg =  new BufferedImage(OrgSizeX, OrgSizeY, BufferedImage.TYPE_3BYTE_BGR);
	private BufferedImage reducededImg = new BufferedImage(DestSizeX, DestSizeY, BufferedImage.TYPE_3BYTE_BGR);
	private BufferedImage lensImg = new BufferedImage(lenssize, lenssize, BufferedImage.TYPE_3BYTE_BGR);
	
	private JPanel controlPanel = new JPanel();
	private JSlider focusSlider = new JSlider(50, 400, 200);

	private boolean loaded = false;
	public static void main(String[] args) {
		
		// get image filename from command line
		if (args.length < 1) {
			System.out.println("Specify filename of picture as command line parameter.");
			System.exit(1);
		}
		String fileName = args[0];
		if (fileName.equals("")) {
			System.out.println("Specify filename of picture as command line parameter.");
			System.exit(1);
		}
		imgPath = fileName;
		
		// create and show window
		ImageFrame frame = new ImageFrame();
		frame.show();
	}
	
	public ImageFrame() {
		// init UI
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		setVisible(true);

		// load image
		Image tmpImg = java.awt.Toolkit.getDefaultToolkit().createImage(imgPath);
		if (tmpImg == null) 
			System.out.println("Error: could not load image");
		setIconImage(tmpImg);

		// scale down image
		OrgSizeX = tmpImg.getWidth(this);
		OrgSizeY = tmpImg.getHeight(this);
		DestSizeX = 600;
		DestSizeY = OrgSizeY * DestSizeX / OrgSizeX;
		originalImg =  new BufferedImage(OrgSizeX, OrgSizeY, BufferedImage.TYPE_3BYTE_BGR);
		reducededImg = new BufferedImage(DestSizeX, DestSizeY, BufferedImage.TYPE_3BYTE_BGR);

		// trick to easily convert Image into a BufferedImage by
		// drawing the Image into the BufferedImage
		originalImg.getGraphics().drawImage(tmpImg, 0, 0, this);
		reducededImg.getGraphics().drawImage(tmpImg, 0, 0, DestSizeX, DestSizeY, this);

		setSize(DestSizeX + 8, DestSizeY + 20);
		ImagePanel panel = new ImagePanel();
		getContentPane().add(panel, BorderLayout.CENTER);
		getContentPane().add(controlPanel, BorderLayout.SOUTH);
		initSlider();
		initRadioButtons();
		
		// add mouse listener for image only
		panel.addMouseListener(this);	
	}
	
	private void initSlider() {
		focusSlider.setValue(lenssize);
		focusSlider.setMinorTickSpacing(10);
		focusSlider.setMajorTickSpacing(100);
		focusSlider.setPaintTicks(true);
		focusSlider.setPaintLabels(true);
		focusSlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				JSlider source = (JSlider) e.getSource();
				if (!source.getValueIsAdjusting()) {
					repaint();
				}
			}

		});
		controlPanel.add(focusSlider);
	}
	
	private void initRadioButtons() {
		JRadioButton magButton = new JRadioButton("Magnify");
		JRadioButton fishButton = new JRadioButton("Fisheye");
		magButton.setActionCommand("Magnify");
		fishButton.setActionCommand("Fisheye");
		magButton.setSelected(true);
		
		ActionListener buttonListener = new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(e.getActionCommand().equals("Fisheye")) {
					fisheye = true;
				} 
				else {
					fisheye = false;
				} 
				repaint();
			}
		};
		
		magButton.addActionListener(buttonListener);
		fishButton.addActionListener(buttonListener);
		ButtonGroup bGroup = new ButtonGroup();
		bGroup.add(magButton);
		controlPanel.add(magButton);
		bGroup.add(fishButton);
		controlPanel.add(fishButton);		
	}

    // mouse events
	public void mousePressed(MouseEvent e) {
    	Point p = e.getPoint();
    	System.out.println("Press: " + p.x + ", " + p.y);
    	System.out.println(e.paramString());
    	
    	if (e.getButton() == 1) {
    		// left mouse button
	    	focusX = p.x;
	    	focusY = p.y;
	    	showLens = true;
    	}
    	else {
    		// any other mouse button
    		showLens = false;
    	}
    	repaint();
	}

    public void mouseClicked(MouseEvent e) {
    }

    public void mouseReleased(MouseEvent e) {
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

  

	class ImagePanel extends JPanel {
		
		public void paint(Graphics g) {
			g.drawImage(reducededImg, 0, 0, this);
			if (showLens) {
				lenssize = focusSlider.getValue();
				lensImg = new BufferedImage(lenssize, lenssize, BufferedImage.TYPE_3BYTE_BGR);
				calcLens();
				g.drawImage(lensImg, focusX - (lenssize>>1), focusY - (lenssize>>1), this);
			}
		}
		
		public void calcLens() {
			//System.out.println("calcLens " + lensImg.getWidth() + ", " + lensImg.getHeight());
			int x, y;
			
			for (int i=0; i<lensImg.getWidth(); i++) {
				for (int j=0; j<lensImg.getHeight(); j++) {
					
					// do your calculations here !!!
					if (fisheye) {
						x = i;
						y = j;
					}
					else {	// magnify					
						x = i;
						y = j;
					}
					int rgbValue = originalImg.getRGB(x, y);
					lensImg.setRGB(i, j, rgbValue);
				}
			}
		}
	}
}
